; Convenient execution of Scheme files from Emacs.
; Top level functions: laml-process-current-buffer, laml-process-buffer,
; and laml-sync-process-current-buffer. 

; The LAML library and programs are written by Kurt Normark, Aalborg University, Denmark.
; Copyright (C) 1999  Kurt Normark, normark@cs.auc.dk.
; 
; This program is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
; 
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
; 
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

(defun laml-process-current-buffer ()
  "Process the current buffer by SCM. The current buffer is supposed to be a laml buffer.
  Before processing, save the current buffer."
  (interactive)
  (let* ((original-buffer (current-buffer))
         (cur-dir (current-directory))
         (file-path (concat cur-dir (current-file)))
         (buf-name-1 (buffer-name (current-buffer)))
         (buf-name
            (if (buffer-versioned buf-name-1)
                (buffer-without-version buf-name-1)
                buf-name-1))
         (filename-without-extension (file-name-proper buf-name))
         (out-buf (get-buffer-create "scm-output"))
         (laml-dir-string (concat "'" "(define laml-dir " (string-it laml-dir) ")"  "'"))
        )
    (save-buffer)
    (delete-other-windows)
    (split-window)
    (show-buffer (other-window 1) out-buf)
    (set-buffer out-buf)
    (erase-buffer)
    (insert "LAML Emacs Processing") (insert CR)
    (goto-char (point-max))
    (set-buffer original-buffer)
    (start-process "LAML" out-buf "/bin/csh"  "-c" 
      (concat scheme-exec " -v " " -e" laml-dir-string " -f " file-path " " filename-without-extension " " cur-dir)
    )
    (other-window 1)
    (enlarge-window source-window-enlargement)
 ))


(defun laml-process-buffer (buffer)
  "Process the laml buffer by SCM. Buffer is the name of a buffer (a string) or a buffer. 
Save buffer before processing"
  (interactive "bWhich buffer: ")
  (let* ((buf-name (if (bufferp buffer) (buffer-name buffer) buffer))
         (the-buffer (if (bufferp buffer) buffer (get-buffer buffer)))
         (buf-name-1
            (if (buffer-versioned buf-name)
                (buffer-without-version buf-name)
                buf-name))
         (out-buf (get-buffer-create "scm-output"))
         (buffer-before (current-buffer))
         (laml-dir-string (concat "'" "(define laml-dir " (string-it laml-dir) ")"  "'"))
        )
    (set-buffer out-buf) (erase-buffer) (insert "LAML Emacs Processing") (insert CR)  (goto-char (point-max))
    (set-buffer buffer)
    (save-buffer)
    (let* ((file-path (buffer-file-name the-buffer))     ; full path of buffer's file
           (buf-dir  (file-name-directory file-path))    ; directory part of file-path
          )   
      (start-process "LAML" out-buf "/bin/csh"  "-c" 
        (concat scheme-exec " -v " " -e" laml-dir-string  " -f " file-path " " (file-name-sans-extension buf-name-1)  " " buf-dir)
      ))
    (display-buffer out-buf)
    (set-buffer buffer-before)
 ))

(defun laml-sync-process-current-buffer ()
  "Process the current buffer by SCM. The current buffer is supposed to be a laml buffer.
   Wait until finished. Before processing, save the current buffer."
  (interactive)
  (let* ((cur-dir (current-directory))
         (scm-output (get-buffer-create "sync-scm-output"))
         (file-path (concat (current-directory) (current-file)))
         (buf-name-1 (buffer-name (current-buffer)))
         (buf-name
             (if (buffer-versioned buf-name-1)
                 (buffer-without-version buf-name-1)
                 buf-name-1))
         (filename-without-extension (file-name-proper buf-name))
         (laml-dir-string (concat "'" "(define laml-dir " (string-it laml-dir) ")"  "'"))
       )
    (save-buffer)
    (save-excursion
      (set-buffer scm-output)
      (erase-buffer))
    (message "Synchronous LAML processing. Please wait...")
    (call-process "/bin/csh" nil scm-output nil "-c" 
       (concat scheme-exec " -v " " -e" laml-dir-string  " -f " file-path " " (file-name-sans-extension buf-name-1) " " cur-dir))
    (message (last-buffer-line scm-output))))




; Ad hoc function used on Unix/SCM only:
(defun run-scheme-on-file (file-name)
  "Run Scheme silently on FILE-NAME"
   (call-process "/bin/csh" nil (get-buffer-create "scm-output") t "-c" 
     (concat scheme-exec " -v "" -f " " " file-name)))






